@@ -17,8 +17,12 @@
 
 package org.apache.commons.pool2.impl;
 
+import java.io.BufferedOutputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.PrintWriter;
 import java.lang.management.ManagementFactory;
 import java.util.ArrayList;
+import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicInteger;
 
@@ -35,8 +39,6 @@ import org.junit.Test;
 
 /**
  * TestCase for AbandonedObjectPool
- *
- * @version $Revision: 1158659 $ $Date: 2011-08-17 05:37:26 -0700 (Wed, 17 Aug 2011) $
  */
 public class TestAbandonedObjectPool {
     private GenericObjectPool<PooledTestObject> pool = null;
@@ -51,28 +53,28 @@ public class TestAbandonedObjectPool {
 
         abandonedConfig.setRemoveAbandonedOnBorrow(true);
         abandonedConfig.setRemoveAbandonedTimeout(1);
-        pool = new GenericObjectPool<PooledTestObject>(
+        pool = new GenericObjectPool<>(
                new SimpleFactory(),
-               new GenericObjectPoolConfig(),
+               new GenericObjectPoolConfig<PooledTestObject>(),
                abandonedConfig);
     }
 
     @After
     public void tearDown() throws Exception {
-        String poolName = pool.getJmxName().toString();
+        final String poolName = pool.getJmxName().toString();
         pool.clear();
         pool.close();
         pool = null;
 
-        MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
-        Set<ObjectName> result = mbs.queryNames(new ObjectName(
+        final MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
+        final Set<ObjectName> result = mbs.queryNames(new ObjectName(
                 "org.apache.commoms.pool2:type=GenericObjectPool,*"), null);
         // There should be no registered pools at this point
-        int registeredPoolCount = result.size();
-        StringBuilder msg = new StringBuilder("Current pool is: ");
+        final int registeredPoolCount = result.size();
+        final StringBuilder msg = new StringBuilder("Current pool is: ");
         msg.append(poolName);
         msg.append("  Still open pools are: ");
-        for (ObjectName name : result) {
+        for (final ObjectName name : result) {
             // Clean these up ready for the next test
             msg.append(name.toString());
             msg.append(" created via\n");
@@ -84,10 +86,11 @@ public class TestAbandonedObjectPool {
     }
 
     /**
-    * Tests fix for Bug 28579, a bug in AbandonedObjectPool that causes numActive to go negative
-    * in GenericObjectPool
-    *
-    */
+     * Tests fix for Bug 28579, a bug in AbandonedObjectPool that causes numActive to go negative
+     * in GenericObjectPool
+     *
+     * @throws Exception May occur in some failure modes
+     */
     @Test
     public void testConcurrentInvalidation() throws Exception {
         final int POOL_SIZE = 30;
@@ -96,19 +99,19 @@ public class TestAbandonedObjectPool {
         pool.setBlockWhenExhausted(false);
 
         // Exhaust the connection pool
-        ArrayList<PooledTestObject> vec = new ArrayList<PooledTestObject>();
+        final ArrayList<PooledTestObject> vec = new ArrayList<>();
         for (int i = 0; i < POOL_SIZE; i++) {
             vec.add(pool.borrowObject());
         }
 
         // Abandon all borrowed objects
-        for (int i = 0; i < vec.size(); i++) {
-            vec.get(i).setAbandoned(true);
+        for (final PooledTestObject element : vec) {
+            element.setAbandoned(true);
         }
 
         // Try launching a bunch of borrows concurrently.  Abandoned sweep will be triggered for each.
         final int CONCURRENT_BORROWS = 5;
-        Thread[] threads = new Thread[CONCURRENT_BORROWS];
+        final Thread[] threads = new Thread[CONCURRENT_BORROWS];
         for (int i = 0; i < CONCURRENT_BORROWS; i++) {
             threads[i] = new ConcurrentBorrower(vec);
             threads[i].start();
@@ -120,8 +123,7 @@ public class TestAbandonedObjectPool {
         }
 
         // Return all objects that have not been destroyed
-        for (int i = 0; i < vec.size(); i++) {
-            PooledTestObject pto = vec.get(i);
+        for (final PooledTestObject pto : vec) {
             if (pto.isActive()) {
                 pool.returnObject(pto);
             }
@@ -135,6 +137,8 @@ public class TestAbandonedObjectPool {
      * Verify that an object that gets flagged as abandoned and is subsequently returned
      * is destroyed instead of being returned to the pool (and possibly later destroyed
      * inappropriately).
+     *
+     * @throws Exception May occur in some failure modes
      */
     @Test
     public void testAbandonedReturn() throws Exception {
@@ -142,9 +146,9 @@ public class TestAbandonedObjectPool {
         abandonedConfig.setRemoveAbandonedOnBorrow(true);
         abandonedConfig.setRemoveAbandonedTimeout(1);
         pool.close();  // Unregister pool created by setup
-        pool = new GenericObjectPool<PooledTestObject>(
+        pool = new GenericObjectPool<>(
                 new SimpleFactory(200, 0),
-                new GenericObjectPoolConfig(), abandonedConfig);
+                new GenericObjectPoolConfig<PooledTestObject>(), abandonedConfig);
         final int n = 10;
         pool.setMaxTotal(n);
         pool.setBlockWhenExhausted(false);
@@ -152,11 +156,9 @@ public class TestAbandonedObjectPool {
         for (int i = 0; i < n - 2; i++) {
             obj = pool.borrowObject();
         }
-        if (obj == null) {
-            throw new NullPointerException("Unable to borrow object from pool");
-        }
+        Objects.requireNonNull(obj, "Unable to borrow object from pool");
         final int deadMansHash = obj.hashCode();
-        ConcurrentReturner returner = new ConcurrentReturner(obj);
+        final ConcurrentReturner returner = new ConcurrentReturner(obj);
         Thread.sleep(2000);  // abandon checked out instances
         // Now start a race - returner waits until borrowObject has kicked
         // off removeAbandoned and then returns an instance that borrowObject
@@ -170,6 +172,8 @@ public class TestAbandonedObjectPool {
     /**
      * Verify that an object that gets flagged as abandoned and is subsequently
      * invalidated is only destroyed (and pool counter decremented) once.
+     *
+     * @throws Exception May occur in some failure modes
      */
     @Test
     public void testAbandonedInvalidate() throws Exception {
@@ -177,10 +181,10 @@ public class TestAbandonedObjectPool {
         abandonedConfig.setRemoveAbandonedOnMaintenance(true);
         abandonedConfig.setRemoveAbandonedTimeout(1);
         pool.close();  // Unregister pool created by setup
-        pool = new GenericObjectPool<PooledTestObject>(
+        pool = new GenericObjectPool<>(
                 // destroys take 200 ms
                 new SimpleFactory(200, 0),
-                new GenericObjectPoolConfig(), abandonedConfig);
+                new GenericObjectPoolConfig<PooledTestObject>(), abandonedConfig);
         final int n = 10;
         pool.setMaxTotal(n);
         pool.setBlockWhenExhausted(false);
@@ -199,6 +203,8 @@ public class TestAbandonedObjectPool {
     /**
      * Verify that an object that the evictor identifies as abandoned while it
      * is in process of being returned to the pool is not destroyed.
+     *
+     * @throws Exception May occur in some failure modes
      */
     @Test
     public void testRemoveAbandonedWhileReturning() throws Exception {
@@ -206,10 +212,10 @@ public class TestAbandonedObjectPool {
         abandonedConfig.setRemoveAbandonedOnMaintenance(true);
         abandonedConfig.setRemoveAbandonedTimeout(1);
         pool.close();  // Unregister pool created by setup
-        pool = new GenericObjectPool<PooledTestObject>(
+        pool = new GenericObjectPool<>(
              // validate takes 1 second
-                new SimpleFactory(0, 1000),
-                new GenericObjectPoolConfig(), abandonedConfig);
+             new SimpleFactory(0, 1000),
+             new GenericObjectPoolConfig<PooledTestObject>(), abandonedConfig);
         final int n = 10;
         pool.setMaxTotal(n);
         pool.setBlockWhenExhausted(false);
@@ -218,10 +224,10 @@ public class TestAbandonedObjectPool {
         // Borrow an object, wait long enough for it to be abandoned
         // then arrange for evictor to run while it is being returned
         // validation takes a second, evictor runs every 500 ms
-        PooledTestObject obj = pool.borrowObject();
+        final PooledTestObject obj = pool.borrowObject();
         Thread.sleep(50);       // abandon obj
         pool.returnObject(obj); // evictor will run during validation
-        PooledTestObject obj2 = pool.borrowObject();
+        final PooledTestObject obj2 = pool.borrowObject();
         Assert.assertEquals(obj, obj2);          // should get original back
         Assert.assertTrue(!obj2.isDestroyed());  // and not destroyed
     }
@@ -232,6 +238,8 @@ public class TestAbandonedObjectPool {
      * further object which should block until the abandoned objects are
      * removed. We don't want the test to block indefinitely when it fails so
      * use maxWait be check we don't actually have to wait that long.
+     *
+     * @throws Exception May occur in some failure modes
      */
     @Test
     public void testWhenExhaustedBlock() throws Exception {
@@ -242,21 +250,43 @@ public class TestAbandonedObjectPool {
         pool.setMaxTotal(1);
 
         @SuppressWarnings("unused") // This is going to be abandoned
+        final
         PooledTestObject o1 = pool.borrowObject();
 
-        long start = System.currentTimeMillis();
-        PooledTestObject o2 = pool.borrowObject(5000);
-        long end = System.currentTimeMillis();
+        final long start = System.currentTimeMillis();
+        final PooledTestObject o2 = pool.borrowObject(5000);
+        final long end = System.currentTimeMillis();
 
         pool.returnObject(o2);
 
         Assert.assertTrue (end - start < 5000);
     }
 
-    class ConcurrentBorrower extends Thread {
-        private ArrayList<PooledTestObject> _borrowed;
+    /**
+     * JIRA: POOL-300
+     */
+    @Test
+    public void testStackTrace() throws Exception {
+        abandonedConfig.setRemoveAbandonedOnMaintenance(true);
+        abandonedConfig.setLogAbandoned(true);
+        abandonedConfig.setRemoveAbandonedTimeout(1);
+        final ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        final BufferedOutputStream bos = new BufferedOutputStream(baos);
+        final PrintWriter pw = new PrintWriter(bos);
+        abandonedConfig.setLogWriter(pw);
+        pool.setAbandonedConfig(abandonedConfig);
+        pool.setTimeBetweenEvictionRunsMillis(100);
+        final PooledTestObject o1 = pool.borrowObject();
+        Thread.sleep(2000);
+        Assert.assertTrue(o1.isDestroyed());
+        bos.flush();
+        Assert.assertTrue(baos.toString().indexOf("Pooled object") >= 0);
+    }
 
-        public ConcurrentBorrower(ArrayList<PooledTestObject> borrowed) {
+    class ConcurrentBorrower extends Thread {
+        private final ArrayList<PooledTestObject> _borrowed;
+
+        public ConcurrentBorrower(final ArrayList<PooledTestObject> borrowed) {
             _borrowed = borrowed;
         }
 
@@ -264,15 +294,15 @@ public class TestAbandonedObjectPool {
         public void run() {
             try {
                 _borrowed.add(pool.borrowObject());
-            } catch (Exception e) {
+            } catch (final Exception e) {
                 // expected in most cases
             }
         }
     }
 
     class ConcurrentReturner extends Thread {
-        private PooledTestObject returned;
-        public ConcurrentReturner(PooledTestObject obj) {
+        private final PooledTestObject returned;
+        public ConcurrentReturner(final PooledTestObject obj) {
             returned = obj;
         }
         @Override
@@ -280,7 +310,7 @@ public class TestAbandonedObjectPool {
             try {
                 sleep(20);
                 pool.returnObject(returned);
-            } catch (Exception e) {
+            } catch (final Exception e) {
                 // ignore
             }
         }
@@ -296,38 +326,38 @@ public class TestAbandonedObjectPool {
             validateLatency = 0;
         }
 
-        public SimpleFactory(long destroyLatency, long validateLatency) {
+        public SimpleFactory(final long destroyLatency, final long validateLatency) {
             this.destroyLatency = destroyLatency;
             this.validateLatency = validateLatency;
         }
 
         @Override
         public PooledObject<PooledTestObject> makeObject() {
-            return new DefaultPooledObject<PooledTestObject>(new PooledTestObject());
+            return new DefaultPooledObject<>(new PooledTestObject());
         }
 
         @Override
-        public boolean validateObject(PooledObject<PooledTestObject> obj) {
+        public boolean validateObject(final PooledObject<PooledTestObject> obj) {
             try {
                 Thread.sleep(validateLatency);
-            } catch (Exception ex) {
+            } catch (final Exception ex) {
                 // ignore
             }
             return true;
         }
 
         @Override
-        public void activateObject(PooledObject<PooledTestObject> obj) {
+        public void activateObject(final PooledObject<PooledTestObject> obj) {
             obj.getObject().setActive(true);
         }
 
         @Override
-        public void passivateObject(PooledObject<PooledTestObject> obj) {
+        public void passivateObject(final PooledObject<PooledTestObject> obj) {
             obj.getObject().setActive(false);
         }
 
         @Override
-        public void destroyObject(PooledObject<PooledTestObject> obj) throws Exception {
+        public void destroyObject(final PooledObject<PooledTestObject> obj) throws Exception {
             obj.getObject().setActive(false);
             // while destroying instances, yield control to other threads
             // helps simulate threading errors
@@ -345,13 +375,13 @@ class PooledTestObject implements TrackedUse {
     private boolean destroyed = false;
     private int _hash = 0;
     private boolean _abandoned = false;
-    private static AtomicInteger hash = new AtomicInteger();
+    private static final AtomicInteger hash = new AtomicInteger();
 
     public PooledTestObject() {
         _hash = hash.incrementAndGet();
     }
 
-    public synchronized void setActive(boolean b) {
+    public synchronized void setActive(final boolean b) {
         active = b;
     }
 
@@ -372,7 +402,7 @@ class PooledTestObject implements TrackedUse {
         return _hash;
     }
 
-    public void setAbandoned(boolean b) {
+    public void setAbandoned(final boolean b) {
         _abandoned = b;
     }
 
@@ -382,15 +412,16 @@ class PooledTestObject implements TrackedUse {
             // Abandoned object sweep will occur no matter what the value of removeAbandonedTimeout,
             // because this indicates that this object was last used decades ago
             return 1;
-        } else {
-            // Abandoned object sweep won't clean up this object
-            return 0;
         }
+        // Abandoned object sweep won't clean up this object
+        return 0;
     }
 
     @Override
-    public boolean equals(Object obj) {
-        if (!(obj instanceof PooledTestObject)) return false;
+    public boolean equals(final Object obj) {
+        if (!(obj instanceof PooledTestObject)) {
+            return false;
+        }
         return obj.hashCode() == hashCode();
     }
 }
